//+------------------------------------------------------------------+
//|                                Jim Sloman's natural market river |
//|                   with Standard Deviation and historical extreme |
//|                                                                  |
//|                                            ocn nmr & SD hist.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DimGray
#property indicator_color2  LimeGreen
#property indicator_color3  Green
#property indicator_color4  DeepSkyBlue
#property indicator_style1  STYLE_DOT
#property indicator_style2  STYLE_DOT

//
//
//
//
//

extern int    NMR.Period             = 40;
extern int    NMR.Price              = PRICE_CLOSE;
extern int    TEMA.Period            = 10;
extern bool   Show.StandardDeviation = true;
extern int    SD.Period              = 30;
extern double SD.Up                  = 2.0;
extern double SD.Down                = 2.0;
extern bool   Show.HistoricalExtreme = true;
extern int    History.LookBack       = 15;

//
//
//
//
//

double nmr[];
double zeroLine[];
double sd[];
double hist[];
double tBuffer[][4];
double alpha;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,zeroLine); SetIndexLabel(0,NULL);
   SetIndexBuffer(1,hist); SetIndexDrawBegin(1,NMR.Period+SD.Period);
   SetIndexBuffer(2,sd);   SetIndexDrawBegin(2,NMR.Period+SD.Period);
   SetIndexBuffer(3,nmr);  SetIndexDrawBegin(3,NMR.Period);
   
   //
   //
   //
   //
   //
   
      TEMA.Period = MathMax(TEMA.Period,1);
      NMR.Period  = MathMax(NMR.Period ,1);
           alpha  = 2.0 /(1.0 + TEMA.Period);

   //
   //
   //
   //
   //

   string PriceType;
      switch(NMR.Price)
      {
         case PRICE_CLOSE:    PriceType = "Close";    break;  // 0
         case PRICE_OPEN:     PriceType = "Open";     break;  // 1
         case PRICE_HIGH:     PriceType = "High";     break;  // 2
         case PRICE_LOW:      PriceType = "Low";      break;  // 3
         case PRICE_MEDIAN:   PriceType = "Median";   break;  // 4
         case PRICE_TYPICAL:  PriceType = "Typical";  break;  // 5
         case PRICE_WEIGHTED: PriceType = "Weighted"; break;  // 6
      }      
   
   //
   //
   //
   //
   //
   
   IndicatorShortName ("nmr ("+NMR.Period+","+PriceType+","+TEMA.Period+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define iPrc 3

//
//
//
//
//

int start()
{
   int    counted_bars=IndicatorCounted();
   int    i,k,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = Bars-counted_bars;
         if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);

   //
   //
   //
   //
   //

   for(i=limit, r=Bars-i-1; i >= 0; i--,r++)
   {
      double rawPrice  = iMA(NULL,0,1,0,MODE_SMA,NMR.Price,i);
      double currPrice = iTema(rawPrice,r);
         if (currPrice > 0)
               tBuffer[r][iPrc] = MathLog(currPrice);
         else  tBuffer[r][iPrc] = 0.00;
      
         //
         //
         //
         //
         //
         
         nmr[i]      = iNmrFunction(1,i);
         zeroLine[i] = 0;
         
         //
         //
         //
         //
         //

         if (Show.StandardDeviation)
         {         
            sd[i] = 0;
            double avg = iSma(nmr,SD.Period,i);
            double deviation = iDeviation(nmr,avg,SD.Period,i);
               if (nmr[i] > 0) sd[i] = avg+deviation*SD.Up;
               if (nmr[i] < 0) sd[i] = avg-deviation*SD.Down;
         }
         
         //
         //
         //
         //
         //
         
         if (Show.HistoricalExtreme)
         {
            hist[i]  = hist[i+1];
               if (nmr[i] >  0) hist[i] = MathMax(0,nmr[ArrayMaximum(nmr,History.LookBack,i)]);
               if (nmr[i] <  0) hist[i] = MathMin(0,nmr[ArrayMinimum(nmr,History.LookBack,i)]);
               if (nmr[i] == 0) hist[i] = 0;
         }               
   }
   
   //
   //
   //
   //
   //
   
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//    natural market river
//
//
//

double iNmrFunction(int startPeriod, int tPos)
{
   int    endPeriod = startPeriod*NMR.Period;
   int    pos       = Bars-tPos-1;
   double sum       = 0.00;
      
   for (int i=startPeriod; i <= endPeriod; i+=startPeriod)
          sum += (tBuffer[pos-i+startPeriod][iPrc]-tBuffer[pos-i][iPrc])*(MathSqrt(i)-MathSqrt(i-startPeriod));
   return(sum*1000.00);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iDeviation(double& array[],double dMA, int period, int pos)
{
   double dSum = 0;
         for(int i=0; i<period; i++,pos++) dSum += (array[pos]-dMA)*(array[pos]-dMA);
   return(MathSqrt(dSum/period));
}

//
//
//
//
//

double iSma(double& array[], int period, int pos)
{
   double sum = 0;
         for(int i=0; i<period; i++,pos++) sum += array[pos];
   return(sum/period);
}

//
//
//
//
//

double iTema(double price,int i)
{
   if (i < 1)
      {
         tBuffer[i][0] = price;
         tBuffer[i][1] = price;
         tBuffer[i][2] = price;
      }
   else
      {
         tBuffer[i][0] = tBuffer[i-1][0]+alpha*(price        -tBuffer[i-1][0]);
         tBuffer[i][1] = tBuffer[i-1][1]+alpha*(tBuffer[i][0]-tBuffer[i-1][1]);
         tBuffer[i][2] = tBuffer[i-1][2]+alpha*(tBuffer[i][1]-tBuffer[i-1][2]);
      }
   return(3*tBuffer[i][0] - 3*tBuffer[i][1] + tBuffer[i][2]);
}